/*
 * JBoss, the OpenSource WebOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package javax.xml.soap;

import java.util.Iterator;

import org.w3c.dom.Element;

/** An object representing an element of a SOAP message that is allowed but not
 * specifically prescribed by a SOAP specification. This interface serves as the
 * base interface for those objects that are specifically prescribed by a SOAP
 * specification.
 * 
 * Methods in this interface that are required to return SAAJ specific objects
 * may "silently" replace nodes in the tree as required to successfully return
 * objects of the correct type. See getChildElements() and javax.xml.soap for
 * details.
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.1 $
 */
public interface SOAPElement
   extends Node, Element
{
   public abstract SOAPElement addAttribute(Name name, String value)
      throws SOAPException;

   public abstract SOAPElement addChildElement(String name) throws SOAPException;
   public abstract SOAPElement addChildElement(String localName, String prefix)
      throws SOAPException;
   public abstract SOAPElement addChildElement(String localName, String prefix,
      String uri) throws SOAPException;
   public abstract SOAPElement addChildElement(Name name) throws SOAPException;
   public abstract SOAPElement addChildElement(SOAPElement child)
      throws SOAPException;

   public abstract SOAPElement addNamespaceDeclaration(String prefix,
      String uri) throws SOAPException;
   public abstract SOAPElement addTextNode(String text) throws SOAPException;
   public abstract Iterator getAllAttributes();
   public abstract String getAttributeValue(Name name);
   public abstract Iterator getChildElements();
   public abstract Iterator getChildElements(Name name);
   public abstract Name getElementName();
   public abstract String getEncodingStyle();
   public abstract Iterator getNamespacePrefixes();
   public abstract String getNamespaceURI(String prefix);
   public abstract Iterator getVisibleNamespacePrefixes();

   public abstract boolean removeAttribute(Name name);
   public abstract void removeContents();
   public abstract boolean removeNamespaceDeclaration(String prefix);
   public abstract void setEncodingStyle(String encodingStyle)
      throws SOAPException;

}
