/*
 * JBoss, the OpenSource EJB server
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package javax.xml.soap;

/**
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.2 $
 */
public abstract class SOAPFactory
{
   private static final String DEFAULT_FACTORY =
      "org.apache.axis.soap.SOAPFactoryImpl";

	public SOAPFactory()
   {      
   }

   public static SOAPFactory newInstance() throws SOAPException
   {
      SOAPFactory factory = null;
      String factoryName = null;

      try
      {
         factoryName = System.getProperty("org.apache.axis.soap.SOAPFactoryImpl",
            DEFAULT_FACTORY);
         ClassLoader loader = Thread.currentThread().getContextClassLoader();
         Class factoryClass = loader.loadClass(factoryName);
         factory = (SOAPFactory) factoryClass.newInstance();
      }
      catch(Throwable t)
      {
         throw new SOAPException("Failed to create SOAPConnectionFactory: "+factoryName, t);
      }
      return factory;
   }

	public abstract Detail createDetail() throws SOAPException;
	public abstract SOAPElement createElement(String localName) throws SOAPException;
	public abstract SOAPElement createElement(String localName, String prefix,
      String uri) throws SOAPException;
	public abstract SOAPElement createElement(Name name) throws SOAPException;
	public abstract Name createName(String localName) throws SOAPException;
	public abstract Name createName(String localName, String prefix, String uri)
      throws SOAPException;
}
