/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */
package org.jfox.ejb.connector;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import javax.ejb.EJBObject;

import org.jfox.ejb.EJBObjectId;
import org.jfox.ejb.EJBObjectMethod;
import org.jfox.ejb.MethodHasher;
import org.jfox.ejb.ObjectMethod;

/**
 * еĻĳЭ InvocationHandler ĸ
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public abstract class ContainerInvokerSupport implements ClientContainerInvoker {
    protected EJBObjectId objectId;
    protected ContainerRemote invoker;

    public ContainerInvokerSupport(EJBObjectId objectId, ContainerRemote invoker) {
        this.objectId = objectId;
        this.invoker = invoker;
    }

    public EJBObjectId getObjectId() {
        return objectId;
    }

    public ContainerRemote getInvoker() {
        return invoker;
    }

    /**
     *  DelegateInvocationHandler ת invoke 
     * <p/>
     * ʵͨõ Object 磺toString, hashCode 
     */
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        if(method.equals(ObjectMethod.ToString)) {
            return ((ClientContainerInvoker) Proxy.getInvocationHandler(proxy)).getObjectId().toString();
        }
        else if(method.equals(ObjectMethod.HashCode)) {
            return new Integer(objectId.hashCode());
        }
        else if(method.getName().equals(ObjectMethod.Equals)) {
            Object obj = args[0];
            if(objectId.isHome()) {
                return new Boolean(objectId.equals(((ContainerInvokerSupport) Proxy.getInvocationHandler(proxy)).getObjectId()));
            }
            else if(obj instanceof EJBObject) {
                return invokeBean(proxy, EJBObjectMethod.IsIdentical, args);
            }
            else {
                return Boolean.FALSE;
            }
        }

        if(objectId.isHome()) { // һ EJBHome 
            return invokeHome(proxy, method, args);
        }
        else {
            return invokeBean(proxy, method, args);
        }
    }

    /**
     *  EJBHome ķ
     * ע⣺Ҫע EJBHome 
     * 磺createʱ˷صĲһ EJBObject 
     *  EJBObject  ObjectIdԱֶɴ
     */
    protected abstract Object invokeHome(Object proxy, Method method, Object[] args) throws Exception;

    /**
     * ִ Bean ķ
     */
    protected abstract Object invokeBean(Object proxy, Method method, Object[] args) throws Exception;

    /**
     * õһ method  hashCode Ψһı־һ method
     *
     * @param method
     * @return
     */
    protected static long getMethodHash(Method method) {
        return MethodHasher.getMethodHash(method);
    }

}
