/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb.connector.jrmp;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import javax.ejb.EJBObject;

import org.jfox.ejb.EJBHomeMethod;
import org.jfox.ejb.EJBObjectId;
import org.jfox.ejb.EJBObjectMethod;
import org.jfox.ejb.HandleImpl;
import org.jfox.ejb.Invocation;
import org.jfox.ejb.InvocationImpl;
import org.jfox.ejb.connector.ContainerInvokerSupport;
import org.jfox.ejb.connector.ContainerRemote;

/**
 * the InvocationHander forward client method to ContainerRemote rmi invoker
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class JRMPContainerInvoker extends ContainerInvokerSupport {

    public JRMPContainerInvoker(EJBObjectId objectHash, ContainerRemote invoker) {
        super(objectHash, invoker);
    }

    protected Object invokeHome(Object proxy, Method method, Object[] args) throws Exception {
        Invocation invocation = new InvocationImpl(objectId, new Long(getMethodHash(method)).toString(), args);

        if(method.equals(EJBHomeMethod.GetEJBMetaData)) {
            return invoker.invoke(invocation);
        }
        else if(method.equals(EJBHomeMethod.GetHomeHandle)) {
//      String homeClass = objectId.getHomeInterfName();
//      return new HomeHandleImpl(homeClass);
            return invoker.invoke(invocation);
        }
        else if(method.equals(EJBHomeMethod.RemoveByHandle)) {
            return invoker.invoke(invocation);
        }
        else if(method.equals(EJBHomeMethod.RemoveByPrimaryKey)) {
            return invoker.invoke(invocation);
        }
        else { // create 
            Object obj = invoker.invoke(invocation); // obj  EJBObject  ID
            EJBObjectId objectId = (EJBObjectId) obj;
            ClassLoader ctxLoader = Thread.currentThread().getContextClassLoader();
            // get the EJBObject class name
            String remoteInterfaceName = objectId.getRemoteInterfaceName();
            Class remoteInterface = ctxLoader.loadClass(remoteInterfaceName);
            // 챾ص EJBObject 
            return Proxy.newProxyInstance(ctxLoader,
                    new Class[]{remoteInterface},
                    new JRMPContainerInvoker(objectId, invoker));
        }
    }

    protected Object invokeBean(Object proxy, Method method, Object[] args) throws Exception {
        Invocation invocation = new InvocationImpl(objectId, new Long(getMethodHash(method)).toString(), args);
        if(method.equals(EJBObjectMethod.GetEJBHome)) {
            return invoker.invoke(invocation);
        }
        else if(method.equals(EJBObjectMethod.GetHandle)) {
            ContainerInvokerSupport invocationHandler = (ContainerInvokerSupport) Proxy.getInvocationHandler(proxy);
            return new HandleImpl(invocationHandler);
        }
        else if(method.equals(EJBObjectMethod.GetPrimaryKey)) {
            return invoker.invoke(invocation);
        }
        else if(method.equals(EJBObjectMethod.IsIdentical)) {
            // Ѳ args[0] תΪEJBObject ġObjectIdֻáObjectId Ƚ
            EJBObject ejbObject = (EJBObject) args[0];
            Object _objectId = ((ContainerInvokerSupport) (Proxy.getInvocationHandler(ejbObject))).getObjectId();
            invocation = new InvocationImpl(objectId, new Long(getMethodHash(method)).toString(), new Object[]{_objectId});
            return invoker.invoke(invocation);
        }
        else if(method.equals(EJBObjectMethod.Remove)) {
            return invoker.invoke(invocation);
        }
        else {
            return invoker.invoke(invocation);
        }
    }

}